###############################################################
# Template code to compute stresses in a half plane subject   #
# to a distributed load on its surface.                       #
#                                                             #
# V.A. Yastrebov, CNRS                                        #
# MINES ParisTech, PSL University, Centre des materiaux       #
# Feb 2021                                                    #
#                                                             #
# Licence: CC0                                                #
#                                                             #
###############################################################

import sys
import numpy as np
import scipy.integrate as integrate
import matplotlib.pyplot as plt
import matplotlib
from matplotlib import cm

matplotlib.rcParams['figure.figsize'] =6,5
matplotlib.rcParams.update({'font.size': 14})
plt.rc('text', usetex=True)
plt.rc('font', family='serif')

# Contact pressure is distributed from -a to a
a = 1.
p0 = 10.

# Pressure distribution
def pres(x):
  return p0
# Integrands
def fxx(x,y,s):
  return pres(s) 
def fyy(x,y,s):
  return pres(s) 
def fxy(x,y,s):
  return pres(s) 

# Read arguments
if len(sys.argv) != 3:
  print("I need 2 arguments: 1 - box size (double), 2 - number of output points per side (int).")
  exit(1)

# Box size
L = float(sys.argv[1])
# Number of output points per side
N = int(sys.argv[2])
# Stress arrays
sig_xx = np.zeros((N,N))
sig_yy = np.zeros((N,N))
sig_xy = np.zeros((N,N))

# Discretization
dl = L/float(N)
#ds = 2*a/float(Nint)

output = file("output.data","w")
output.write("# x y sig_xx sig_yy sig_xy\n")
# Loop over y points
y= -L+dl/2.
for i in range(N):
  x=-L/2.+dl/2
# Loop over x points
  for j in range(N):
  
# Evaluate integrals using trapezoidal rule
    sig_xx[i][j] = integrate.quad(lambda s: fxx(x,y,s), -a, a)[0]
    sig_yy[i][j] = integrate.quad(lambda s: fyy(x,y,s), -a, a)[0]
    sig_xy[i][j] = integrate.quad(lambda s: fxy(x,y,s), -a, a)[0]
    output.write(str(x)+" "+str(y)+" "+str(sig_xx[i][j])+" "+str(sig_yy[i][j])+" "+str(sig_xy[i][j])+"\n")       
    x+=dl
  output.write("\n")
  y+=dl

output.close()    

x = np.linspace(-L/2. + dl/2., L/2. - dl/2., N)
y = np.linspace(-L + dl/2., -dl/2., N)
X, Y = np.meshgrid(x, y)
NumColors = 11

# Fig sig_xx
fig,ax = plt.subplots()
plt.xlabel("$x/a$")
plt.ylabel("$y/a$")

lev = np.linspace(np.min(sig_xx/p0),np.max(sig_xx/p0),NumColors)

cs = ax.contourf(X/a, Y/a, sig_xx/p0, levels=lev, cmap=cm.rainbow)
ax.contour(X/a,Y/a,sig_xx/p0,levels=lev,colors="k")
cbar = fig.colorbar(cs)
cbar.ax.set_ylabel("$\sigma_{xx}/p_0$")
fig.savefig("Sigma_xx.pdf")
plt.show()

# Fig sig_xy
fig,ax = plt.subplots()
plt.xlabel("$x/a$")
plt.ylabel("$y/a$")

lev = np.linspace(np.min(sig_xy/p0),np.max(sig_xy/p0),NumColors)

cs = ax.contourf(X/a, Y/a, sig_xy/p0, levels=lev, cmap=cm.rainbow)
ax.contour(X/a,Y/a,sig_xy/p0,levels=lev,colors="k")
cbar = fig.colorbar(cs)
cbar.ax.set_ylabel("$\sigma_{xy}/p_0$")
fig.savefig("Sigma_xy.pdf")
plt.show()

# Fig sig_yy
fig,ax = plt.subplots()
plt.xlabel("$x/a$")
plt.ylabel("$y/a$")

lev = np.linspace(np.min(sig_yy/p0),np.max(sig_yy/p0),NumColors)

cs = ax.contourf(X/a, Y/a, sig_yy/p0, levels=lev, cmap=cm.rainbow)
ax.contour(X/a,Y/a,sig_yy/p0,levels=lev,colors="k")
cbar = fig.colorbar(cs)
cbar.ax.set_ylabel("$\sigma_{yy}/p_0$")
fig.savefig("Sigma_yy.pdf")
plt.show()


